import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.moyosoft.exchange.*;
import com.moyosoft.exchange.contact.*;

public class ExchangeContactDialog extends JDialog
{
	private ExchangeContact mContact;
	private ContactPanel mContactPanel = new ContactPanel(this);
	private boolean mAddingNew;
	
	public ExchangeContactDialog(Frame parent, boolean addingNew)
	{
		super(parent);
		mAddingNew = addingNew;
		init();
	}
	
	protected void init()
	{
		setModal(true);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(BorderLayout.CENTER, mContactPanel);
	}
	
	public void close()
	{
		dispose();
	}
	
	public void load(ExchangeContact contact) throws ExchangeServiceException
	{
		mContact = contact;
		
		if(mContact != null)
		{
			mContactPanel.load(mContact);
		}
		
		contactChanged();
	}
	
	protected void contactChanged() throws ExchangeServiceException
	{
		updateTitle();
	}
	
	private static String notNull(String str)
	{
		if(str == null) return "";
		return str;
	}
	
	protected void updateTitle() throws ExchangeServiceException
	{
		if(mContact != null)
		{
			setTitle("Contact - " + notNull(mContact.getFirstName()) + " " + notNull(mContact.getLastName()));
		}
		else
		{
			setTitle("Contact");
		}
	}

	private void showError(ExchangeServiceException ex)
	{
		ex.printStackTrace();
		
		JOptionPane.showMessageDialog(
				this,
				ex.getMessage(),
				"Error",
				JOptionPane.ERROR_MESSAGE
				);
	}
	
	public void closePressed()
	{
		if(mAddingNew && mContact != null)
		{
			try
			{
				mContact.delete(false);
			}
			catch(ExchangeServiceException ex)
			{
			}
		}
		
		close();
	}
	
	public void savePressed()
	{
		try
		{
			mContactPanel.store(mContact);
			mContact.save();
			
			close();
		}
		catch(ExchangeServiceException ex)
		{
			showError(ex);
		}
	}
	
	private static void centerOnScreen(Window window)
	{
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension windowSize = window.getSize();
		windowSize.height = Math.min(windowSize.height + 20, screenSize.height);
		windowSize.width = Math.min(windowSize.width, screenSize.width);
		window.setLocation((screenSize.width - windowSize.width) / 2,
				(screenSize.height - windowSize.height) / 2);
	}
	
	public static void open(Frame parent, ExchangeContact contact, boolean addingNew) throws ExchangeServiceException
	{
		ExchangeContactDialog dialog = new ExchangeContactDialog(parent, addingNew);
		dialog.pack();
		centerOnScreen(dialog);
		
		dialog.load(contact);
		
		dialog.setVisible(true);
	}
}

class ContactPanel extends JPanel
{
	private ExchangeContactDialog mParent;
	private JButton mButtonClose = new JButton("Cancel");
	private JButton mButtonSave = new JButton("Save");
	private ContactAttributesPanel mContactAttributesPanel = new ContactAttributesPanel();
	
	public ContactPanel(ExchangeContactDialog parent)
	{
		super();
		mParent = parent;
		init();
	}
	
	protected void init()
	{
		createPanel();
	}
	
	protected void createPanel()
	{
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.add(mButtonSave);
		buttonPanel.add(mButtonClose);
		
		setLayout(new BorderLayout());
		add(BorderLayout.CENTER, mContactAttributesPanel);
		add(BorderLayout.SOUTH, buttonPanel);
		
		mContactAttributesPanel.setBorder(BorderFactory.createEmptyBorder(14, 7, 5, 7));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 4, 4, 4));
		
		mButtonClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
				closePressed();
			}
		});
		mButtonSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event)
			{
				savePressed();
			}
		});
	}
	
	public void closePressed()
	{
		mParent.closePressed();
	}
	
	public void savePressed()
	{
		mParent.savePressed();
	}
	
	public void load(ExchangeContact contact) throws ExchangeServiceException
	{
		mContactAttributesPanel.load(contact);
	}
	
	public void store(ExchangeContact contact) throws ExchangeServiceException
	{
		mContactAttributesPanel.store(contact);
	}
}

class ContactAttributesPanel extends JPanel
{
	private JTextField mFieldFirstname = new JTextField();
	private JTextField mFieldLastname = new JTextField();
	private JTextField mFieldCompany = new JTextField();
	private JTextField mFieldHomePhone = new JTextField();
	private JTextField mFieldWorkPhone = new JTextField();
	private JTextField mFieldMobilePhone = new JTextField();
	private JTextField mFieldFaxNumber = new JTextField();
	private JTextField mFieldEmail = new JTextField();
	private JTextField mFieldAddressStreet = new JTextField();
	private JTextField mFieldAddressCity = new JTextField();
	private JTextField mFieldAddressPostalCode = new JTextField();
	private JTextField mFieldAddressState = new JTextField();
	private JTextField mFieldAddressCountry = new JTextField();
	private JTextArea mFieldBody = new JTextArea();
	private ExchangeAttachmentsPanel mFieldAttachments = new ExchangeAttachmentsPanel();
	
	public ContactAttributesPanel()
	{
		super();
		init();
	}
	
	public ContactAttributesPanel(boolean pDoubleBuffered)
	{
		super(pDoubleBuffered);
		init();
	}
	
	protected void init()
	{
		createPanel();
		
		setPreferredSize(new Dimension(580, 360));
	}
	
	protected void createPanel()
	{
		// Create panels
		JPanel panelInfoLeft = createInfoLeftPanel();
		JPanel panelInfoRight = createInfoRightPanel();
		JPanel panelInfoBottomLeft = createInfoBottomLeftPanel();
		JPanel panelInfoAddress = createInfoAddressPanel();
		JPanel panelBody = createBodyPanel();
		JPanel panelAttachements = createAttachementsPanel();
		
		JLabel separator1 = new JLabel("");
		JLabel separator2 = new JLabel("");
		JLabel separator3 = new JLabel("");
		
		// GridBagLayout
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		// insets between components
		Insets separatorInsets = new Insets(10, 3, 10, 3);
		Insets leftInsets = new Insets(0, 3, 0, 10);
		Insets rightInsets = new Insets(0, 10, 0, 3);
		
		// two columns: 50% of the width each
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.NORTHWEST;
		c.weightx = 1;
		
		// 0,0
		c.gridx = 0;
		c.gridy = 0;
		c.insets = leftInsets;
		add(panelInfoLeft, c);
		
		// 1,0
		c.gridx = 1;
		c.gridy = 0;
		c.insets = rightInsets;
		add(panelInfoRight, c);
		
		// separator line: 0,1 and 1,1
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 2;
		c.insets = separatorInsets;
		add(separator1, c);
		c.gridwidth = 1;
		
		// 0,2
		c.gridx = 0;
		c.gridy = 2;
		c.insets = leftInsets;
		add(panelInfoBottomLeft, c);
		
		// 1,2
		c.gridx = 1;
		c.gridy = 2;
		c.insets = rightInsets;
		add(panelInfoAddress, c);
		
		// separator line: 0,3 and 1,3
		c.gridx = 0;
		c.gridy = 3;
		c.gridwidth = 2;
		c.insets = separatorInsets;
		add(separator2, c);
		c.gridwidth = 1;
		
		// 0,4 : fill all remaining height space
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.gridy = 4;
		c.weighty = 1;
		c.insets = leftInsets;
		add(panelBody, c);
		
		// 1,4 : fill all remaining height space
		c.gridx = 1;
		c.gridy = 4;
		c.insets = rightInsets;
		add(panelAttachements, c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weighty = 0;
		
		// separator line: 0,5 and 1,5
		c.gridx = 0;
		c.gridy = 5;
		c.gridwidth = 2;
		c.insets = new Insets(10, 3, 0, 3);
		add(separator3, c);
		c.gridwidth = 1;
	}
	
	private JPanel createInfoLeftPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		labelConstraints.insets = new Insets(0, 0, 0, 7);
		labelConstraints.anchor = GridBagConstraints.WEST;
		
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.insets = new Insets(0, 0, 2, 0);
		fieldConstraints.weightx = 1;
		
		// Last name
		// 0,0
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		panel.add(new JLabel("Last name"), labelConstraints);
		
		// 1,0
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 0;
		panel.add(mFieldLastname, fieldConstraints);
		
		// First name
		// 0,1
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 1;
		panel.add(new JLabel("First name"), labelConstraints);
		
		// 1,1
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 1;
		panel.add(mFieldFirstname, fieldConstraints);
		
		// Company
		// 0,2
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 2;
		panel.add(new JLabel("Company"), labelConstraints);
		
		// 1,2
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 2;
		panel.add(mFieldCompany, fieldConstraints);
		
		return panel;
	}
	
	private JPanel createInfoRightPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		labelConstraints.insets = new Insets(0, 0, 0, 7);
		labelConstraints.anchor = GridBagConstraints.WEST;
		
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.insets = new Insets(0, 0, 2, 0);
		fieldConstraints.weightx = 1;
		
		// E-mail
		// 0,0
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		panel.add(new JLabel("E-mail"), labelConstraints);
		
		// 1,0
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 0;
		panel.add(mFieldEmail, fieldConstraints);
		
		return panel;
	}
	
	private JPanel createInfoBottomLeftPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		labelConstraints.insets = new Insets(0, 0, 0, 7);
		labelConstraints.anchor = GridBagConstraints.WEST;
		
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.insets = new Insets(0, 0, 2, 0);
		fieldConstraints.weightx = 1;
		
		// Home phone
		// 0,0
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		panel.add(new JLabel("Home phone"), labelConstraints);
		
		// 1,0
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 0;
		panel.add(mFieldHomePhone, fieldConstraints);
		
		// Work phone
		// 0,1
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 1;
		panel.add(new JLabel("Work phone"), labelConstraints);
		
		// 1,1
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 1;
		panel.add(mFieldWorkPhone, fieldConstraints);
		
		// Mobile phone
		// 0,2
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 2;
		panel.add(new JLabel("Mobile"), labelConstraints);
		
		// 1,2
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 2;
		panel.add(mFieldMobilePhone, fieldConstraints);
		
		// Fax
		// 0,3
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 3;
		panel.add(new JLabel("Fax"), labelConstraints);
		
		// 1,3
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 3;
		panel.add(mFieldFaxNumber, fieldConstraints);
		
		return panel;
	}
	
	private JPanel createInfoAddressPanel()
	{
		JPanel panel = new JPanel(new GridBagLayout());
		
		GridBagConstraints labelConstraints = new GridBagConstraints();
		labelConstraints.insets = new Insets(0, 0, 0, 7);
		labelConstraints.anchor = GridBagConstraints.WEST;
		
		GridBagConstraints fieldConstraints = new GridBagConstraints();
		fieldConstraints.fill = GridBagConstraints.HORIZONTAL;
		fieldConstraints.insets = new Insets(0, 0, 2, 0);
		fieldConstraints.weightx = 1;
		
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 0;
		panel.add(new JLabel("Street"), labelConstraints);
		
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 0;
		panel.add(mFieldAddressStreet, fieldConstraints);
		
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 1;
		panel.add(new JLabel("City"), labelConstraints);
		
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 1;
		panel.add(mFieldAddressCity, fieldConstraints);
		
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 2;
		panel.add(new JLabel("Postal code"), labelConstraints);
		
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 2;
		panel.add(mFieldAddressPostalCode, fieldConstraints);
		
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 3;
		panel.add(new JLabel("State"), labelConstraints);
		
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 3;
		panel.add(mFieldAddressState, fieldConstraints);
		
		labelConstraints.gridx = 0;
		labelConstraints.gridy = 4;
		panel.add(new JLabel("Country"), labelConstraints);
		
		fieldConstraints.gridx = 1;
		fieldConstraints.gridy = 4;
		panel.add(mFieldAddressCountry, fieldConstraints);
		
		return panel;
	}
	
	private JPanel createBodyPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());
		
		panel.add(BorderLayout.NORTH, new JLabel("Body"));
		
		JScrollPane scrollPane = new JScrollPane(mFieldBody);
		panel.add(BorderLayout.CENTER, scrollPane);
		
		panel.setPreferredSize(new Dimension(10, 10));
		
		return panel;
	}
	
	private JPanel createAttachementsPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());
		
		panel.add(BorderLayout.NORTH, new JLabel("Attachements"));
		panel.add(BorderLayout.CENTER, mFieldAttachments);
		
		panel.setPreferredSize(new Dimension(10, 10));
		
		return panel;
	}
	
	public void load(ExchangeContact contact) throws ExchangeServiceException
	{
		if(contact == null)
		{
			return;
		}
		
		mFieldAddressStreet.setText(contact.getAddressHomeStreet());
		mFieldAddressCity.setText(contact.getAddressHomeCity());
		mFieldAddressPostalCode.setText(contact.getAddressHomePostalCode());
		mFieldAddressState.setText(contact.getAddressHomeState());
		mFieldAddressCountry.setText(contact.getAddressHomeCountryOrRegion());
		mFieldBody.setText(contact.getBody());
		mFieldCompany.setText(contact.getCompanyName());
		mFieldEmail.setText(contact.getEmailAddress1());
		mFieldFaxNumber.setText(contact.getPhoneNumberHomeFax());
		mFieldFirstname.setText(contact.getFirstName());
		mFieldHomePhone.setText(contact.getPhoneNumberHomePhone());
		mFieldLastname.setText(contact.getLastName());
		mFieldMobilePhone.setText(contact.getPhoneNumberMobilePhone());
		mFieldWorkPhone.setText(contact.getPhoneNumberBusinessPhone());
		mFieldAttachments.load(contact);
	}
	
	public void store(ExchangeContact contact) throws ExchangeServiceException
	{
		contact.setAddressHome(
				mFieldAddressStreet.getText(),
				mFieldAddressCity.getText(),
				mFieldAddressState.getText(),
				mFieldAddressCountry.getText(),
				mFieldAddressPostalCode.getText());
		
		contact.setBody(mFieldBody.getText());
		contact.setCompanyName(mFieldCompany.getText());
		contact.setEmailAddress1(mFieldEmail.getText());
		contact.setPhoneNumberHomeFax(mFieldFaxNumber.getText());
		contact.setFirstName(mFieldFirstname.getText());
		contact.setPhoneNumberHomePhone(mFieldHomePhone.getText());
		contact.setLastName(mFieldLastname.getText());
		contact.setPhoneNumberMobilePhone(mFieldMobilePhone.getText());
		contact.setPhoneNumberBusinessPhone(mFieldWorkPhone.getText());
	}
}
