package com.moyosoft.exchange.javamail;

import java.io.*;
import java.util.*;

import javax.mail.*;
import javax.mail.event.*;
import javax.mail.internet.*;

import com.moyosoft.exchange.*;
import com.moyosoft.exchange.folder.*;
import com.moyosoft.exchange.item.*;
import com.moyosoft.exchange.mail.*;
import com.sun.mail.util.*;

public class JbexFolder extends Folder
{
	private ExchangeFolder folder;
	private boolean isOpen;
	
	protected JbexFolder(Store store, ExchangeFolder folder)
	{
		super(store);
		this.folder = folder;
	}

	public void appendMessages(Message[] msgs) throws MessagingException
	{
		for(Message msg : msgs)
		{
			appendMessage(msg);
		}
		
		notifyMessageAddedListeners(msgs);
	}

	public void appendMessage(Message msg) throws MessagingException
	{
		if(!(msg instanceof MimeMessage))
		{
			return;
		}

		MimeMessage mimeMessage = (MimeMessage) msg;
		ByteArrayOutputStream mimeBytesStream = new ByteArrayOutputStream();

		try
		{
			mimeMessage.writeTo(new CRLFOutputStream(mimeBytesStream));
		}
		catch(IOException e)
		{
			throw new MessagingException("Unable to write message content", e);
		}

		byte[] mimeContent = mimeBytesStream.toByteArray();

		if(mimeContent == null || mimeContent.length == 0)
		{
			return;
		}

		try
		{
			ExchangeMail mail = folder.createMail();
			mail.setMimeContent(mimeContent);
			mail.save();
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
	}

	public void close(boolean expunge) throws MessagingException
	{
		isOpen = false;
		notifyConnectionListeners(ConnectionEvent.CLOSED);
	}

	public boolean create(int type) throws MessagingException
	{
		// TODO Folder creation
		return false;
	}

	public boolean delete(boolean recurse) throws MessagingException
	{
		try
		{
			folder.delete(true);
			return true;
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
		
		return false;
	}

	public boolean exists() throws MessagingException
	{
		return folder != null;
	}

	public Message[] expunge() throws MessagingException
	{
		// TODO
		throw new UnsupportedOperationException("Deleting is not supported by this implementation");
	}

	public Folder getFolder(String name) throws MessagingException
	{
		if(exists())
		{
			try
			{
				return new JbexFolder(getStore(), folder.getFolders().get(name));
			}
			catch(ExchangeServiceException e)
			{
				handleError(e);
			}
		}
		
		return new JbexFolder(getStore(), null);
	}

	private void buildFullName(StringBuilder builder, ExchangeFolder folder) throws ExchangeServiceException, MessagingException
	{
		if(folder == null)
		{
			return;
		}
		
		buildFullName(builder, folder.getParentFolder());
		
		builder.append(folder.getDisplayName());
		builder.append(getSeparator());
	}

	public String getFullName()
	{
		try
		{
			if(exists())
			{
				StringBuilder builder = new StringBuilder();
				
				buildFullName(builder, folder.getParentFolder());
				builder.append(folder.getDisplayName());
				
				return builder.toString();
			}
		}
		catch(ExchangeServiceException e)
		{
		}
		catch(MessagingException e)
		{
		}
		
		return null;
	}

	private Message createMessage(ExchangeItem item) throws ExchangeServiceException, MessagingException
	{
		// TODO: Create a light-weight message first
		return
			new MimeMessage(
				((JbexStore) store).getSession(),
				new ByteArrayInputStream(item.getMimeContentBytes()
			));
	}
	
	public Message getMessage(int msgnum) throws MessagingException
	{
		if(exists())
		{
			try
			{
				return createMessage(folder.getItems().getAt(msgnum - 1));
			}
			catch(ExchangeServiceException e)
			{
				handleError(e);
			}
		}

		return null;
	}

	public int getMessageCount() throws MessagingException
	{
		try
		{
			return folder.getItemsCount();
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
		
		return 0;
	}

	public String getName()
	{
		try
		{
			if(exists())
			{
				return folder.getDisplayName();
			}
		}
		catch(ExchangeServiceException e)
		{
		}
		catch(MessagingException e)
		{
		}
		
		return null;
	}

	public Folder getParent() throws MessagingException
	{
		if(exists())
		{
			try
			{
				return new JbexFolder(getStore(), folder.getParentFolder());
			}
			catch(ExchangeServiceException e)
			{
				handleError(e);
			}
		}
		
		return new JbexFolder(getStore(), null);
	}

	public Flags getPermanentFlags()
	{
		return null;
	}

	public char getSeparator() throws MessagingException
	{
		return '/';
	}

	public int getType() throws MessagingException
	{
		return HOLDS_FOLDERS | HOLDS_MESSAGES;
	}

	public boolean hasNewMessages() throws MessagingException
	{
		try
		{
			return folder.getUnreadItemsCount() > 0;
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
		
		return false;
	}

	public boolean isOpen()
	{
		return isOpen;
	}

	public Folder[] list(String pattern) throws MessagingException
	{
		// TODO: Use the specified pattern
		
		try
		{
			ArrayList<Folder> folders = new ArrayList<Folder>();
			
			for(ExchangeFolder subfolder : folder.getFolders())
			{
				folders.add(new JbexFolder(getStore(), subfolder));
			}
			
			return folders.toArray(new Folder[folders.size()]);
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
		
		return null;
	}

	public void open(int mode) throws MessagingException
	{
		isOpen = true;
		notifyConnectionListeners(ConnectionEvent.OPENED);
	}

	public boolean renameTo(Folder folder) throws MessagingException
	{
		try
		{
			this.folder.setDisplayName(folder.getName());
			this.folder.save();
			
			notifyFolderRenamedListeners(this);
			
			return true;
		}
		catch(ExchangeServiceException e)
		{
			handleError(e);
		}
		
		return false;
	}

	private void handleError(ExchangeServiceException e) throws MessagingException
	{
		throw new MessagingException(e.toString(), e);
	}
}
